/* Copyright (c) 2015 Douglas Henke <dhenke@mythopoeic.org>
   This is free software -- see COPYING for details.           */
#ifndef ALPHA15X4_H
#define ALPHA15X4_H

/* See README for introduction and licensing information. */

/* See alpha15x4.c for API documentation. */

/* For more details on the commmands, see the HTK16K33 data sheet:
   http://www.adafruit.com/datasheets/ht16K33v110.pdf */

#include <stdint.h>

#define ALPHA15X4_I2C_ADDR 0x70 /* default; change if A0/A1 jumpers shorted */

/* Command bytes. These are all the documented commands for the controller
   chip, some of which are useless in the context of the Adafruit LED
   backpack. The only ones we'll use are DISPLAY_XFER, SYSTEM_SETUP,
   DISPLAY_SETUP and DISPLAY_DIM. */
#define ALPHA15X4_CMD_DISPLAY_XFER  0x00 /* RW; low 4 bits = start addr */
#define ALPHA15X4_CMD_SYSTEM_SETUP  0x20 /* W; low bit = oscillator on/off */
#define ALPHA15X4_CMD_KEYBD_READ    0x40 /* R; low 3 bits = start addr */
#define ALPHA15X4_CMD_INT_FLAG_READ 0x60 /* R; get INT flag state */
#define ALPHA15X4_CMD_DISPLAY_SETUP 0x80 /* W; bit 0=on/off; bits 1-3=blink */
#define ALPHA15X4_CMD_ROW_INT_SETUP 0xA0 /* W; bits 0,1 = state */
#define ALPHA15X4_CMD_DISPLAY_DIM   0xE0 /* W; low 4 bits = brightness */
#define ALPHA15X4_CMD_TEST_MODE     0xD9 /* W; "HOLTEK use only" */

#define ALPHA15X4_PWR_ON 1
#define ALPHA15X4_PWR_STANDBY 0
#define alpha15x4_pwr(x) alpha15x4_cmd(ALPHA15X4_CMD_SYSTEM_SETUP|x)

/* The dimming uses PWM, and is adjustable from 1:16 duty cycle up to
   100% (in steps of 1:16). You can't turn the display completely off
   with the dim command; clear it, set the mode to OFF or set the power
   to standby if you want that. */
#define ALPHA15X4_DIM_1_16 0
#define ALPHA15X4_DIM_2_16 1
#define ALPHA15X4_DIM_3_16 2
#define ALPHA15X4_DIM_4_16 3
#define ALPHA15X4_DIM_5_16 4
#define ALPHA15X4_DIM_6_16 5
#define ALPHA15X4_DIM_7_16 6
#define ALPHA15X4_DIM_8_16 7
#define ALPHA15X4_DIM_9_16 8
#define ALPHA15X4_DIM_10_16 9
#define ALPHA15X4_DIM_11_16 10
#define ALPHA15X4_DIM_12_16 11
#define ALPHA15X4_DIM_13_16 12
#define ALPHA15X4_DIM_14_16 13
#define ALPHA15X4_DIM_15_16 14
#define ALPHA15X4_DIM_16_16 15
#define ALPHA15X4_DIM_FULL ALPHA15X4_DIM_16_16
#define ALPHA15X4_DIM_HALF ALPHA15X4_DIM_8_16
#define alpha15x4_dim(x) alpha15x4_cmd(ALPHA15X4_CMD_DISPLAY_DIM|x)

#define ALPHA15X4_MODE_ON 1
#define ALPHA15X4_MODE_OFF 0
#define ALPHA15X4_MODE_BLINK_2HZ 3
#define ALPHA15X4_MODE_BLINK_1HZ 5
#define ALPHA15X4_MODE_BLINK_HALF_HZ 7
#define alpha15x4_mode(x) alpha15x4_cmd(ALPHA15X4_CMD_DISPLAY_SETUP|x)

/* alpha15x4_init() is a macro to simplify initialization. It clears
   the display memory, turns on the oscillator, sets the brightness,
   and sets the mode to "on" (with no blink). The single argument is
   the desired dimming level -- probably one of the ALPHA15X4_DIM_*
   defines above. */
#define alpha15x4_init(x) \
   alpha15x4_clear();\
   alpha15x4_pwr(ALPHA15X4_PWR_ON);\
   alpha15x4_dim(x);\
   alpha15x4_mode(ALPHA15X4_MODE_ON);

void alpha15x4_cmd(const uint8_t x);
void alpha15x4_clear(void);
void alpha15x4_draw(const uint8_t pos, const char c, const uint8_t dp);

#endif /* ifndef ALPHA15X4_H */
