/* Copyright (c) 2015 Douglas Henke <dhenke@mythopoeic.org>
   This is free software -- see COPYING for details.           */
#ifndef ROTARY_ENCODER_H
#define ROTARY_ENCODER_H

#include <stdint.h>

/* Using the "packed" attribute for an enum is equivalent to using the
   -fshort-enums command-line flag for GCC. In either case, we use the
   narrowest possible integral type that can fit all the enum values
   (i.e., one byte in the case below): */
typedef enum __attribute__((__packed__)) {
   BTN_NONE=0, BTN_DOWN, BTN_UP, BTN_LONG
} button_evt_t;

/* The evt_t structure contains information about user interface events
   that have yet to be handled. Events are set in the interrupt handlers
   (for pin-change and debounce timer expiry, respectively). Events
   are cleared by the main loop as they are handled. (Note that the main
   loop code that modifies _evt must run with interrupts disabled!)

   Unlike most non-embedded UI event systems, this is not an event queue.
   In the case of the button, only the most recent event is recorded. For
   the rotary encoder, a running total position is kept up to +/-127 steps.

   The rotary encoder "steps" counted in this structure are individual
   Gray code changes. The encoder we are using has four changes per
   "click".

   Note that when the user presses and holds the button, you'll get both
   a "down" event (immediately) and a "long press" event (eventually). */
typedef struct {
   button_evt_t btn;
   int8_t rotary; /* positive=CW, negative=CCW */
} evt_t;

extern volatile evt_t _evt;

void renc_setup(void);

#endif /* ifndef ROTARY_ENCODER_H */
