#include <stdint.h>
#include <string.h>
#include <ds1307_rtc/ds1307_rtc.h>
#include "settings.h"

#define SETTINGS_SIGNATURE 0x5A /* any pattern not likely to happen at random */

/*** settings_store() -- store settings structure in persistent storage

Copies structure representing operational settings to persistent storage.

Arguments:
   p -- pointer to settings_t structure to be stored

Note: the signature field may be altered by this function.

Note: the contents of the DS1307_RAM_XFER_FLDS fields may be altered by
this function. Treat DS1307_RAM_XFER_FLDS as opaque.
***/
void settings_store(settings_t * const p) {
   p->signature = SETTINGS_SIGNATURE;
   ds1307_ram_store(p, sizeof(*p));
}

/*** settings_get() -- get settings from persistent storage

Retrieves the settings from persistent storage if possible. Otherwise,
use hardwired conservative defaults.

Arguments:
   p -- pointer to settings_t structure to be populated

Note: the contents of the DS1307_RAM_XFER_FLDS fields following a call to
this function may not match the contents of those fields when settings_store()
was called. Treat DS1307_RAM_XFER_FLDS as opaque.
***/
void settings_get(settings_t * const p) {
   ds1307_ram_fetch(p, sizeof(*p));
   if(p->signature == SETTINGS_SIGNATURE) return;

   /* If the data we just read doesn't have a valid signature, the
      battery-backed RAM may have been reset (and in any case we can't
      trust the settings). Revert to hardwired defaults: */
   memset(p, 0, sizeof(*p));
   p->dim = 8; /* should be legible but not eye-searing */
   p->alarmh = 8; /* marginally better than midnight? */
}
